use std::fmt;
use crate::prelude::Mailbox;
use super::{Link, Recipients};

#[derive(Debug)]
pub struct PreBlk<'a> {
    alt: Option<&'a str>,
    lines: Vec<&'a str>,
}

#[derive(Debug)]
enum State<'a> {
    Normal,
    Preformatted(PreBlk<'a>),
    Quote(Vec<&'a str>),
}

#[derive(Clone, Debug, PartialEq)]
pub enum GemtextNode {
    Sender(Mailbox),
    Recipients(Recipients),
    Timestamp(String),
    Text(String),
    Heading1(String),
    Heading2(String),
    Heading3(String),
    Quote(String),
    Preformatted(String),
    Link(Link),
}

impl fmt::Display for GemtextNode {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Sender(m) => writeln!(f, "< {m}"),
            Self::Recipients(r) => writeln!(f, "{r}"),
            Self::Timestamp(t) => writeln!(f, "@ {t}"),
            Self::Text(t) => writeln!(f, "{t}"),
            Self::Heading1(h) => writeln!(f, "# {h}"),
            Self::Heading2(h) => writeln!(f, "## {h}"),
            Self::Heading3(h) => writeln!(f, "### {h}"),
            Self::Quote(q) => writeln!(f, "> {q}"),
            Self::Preformatted(p) => writeln!(f, "```\n{p}\n```"),
            Self::Link(l) => writeln!(f, "=> {l}"),
        }
    }
}

impl<'a> GemtextNode {
    fn parse_link(text: &'a str) -> Self {
        todo!()
    }

    fn parse_prompt(text: &'a str) -> Self {
        todo!()
    }

    fn parse_heading(text: &'a str) -> Self {
        todo!()
    }

    fn parse_list_item(text: &'a str) -> Self {
        todo!()
    }

    fn parse_blockquote(text: &'a str) -> Self {
        todo!()
    }

    fn parse_senders(text: &'a str) -> Self {
        todo!()
    }

    fn parse_recipients(text: &'a str) -> Self {
        todo!()
    }

    fn parse_timestamp(text: &'a str) -> Self {
        todo!()
    }
}

#[derive(Debug)]
pub struct Parser<'a> {
    state: State<'a>,
    lines: Vec<GemtextNode>,
}

impl<'a> Parser<'a> {
    pub fn new() -> Self {
        Self {
            state: State::Normal,
            lines: vec![]
        }
    }

    fn parse(mut self, raw: &'a str) -> Vec<GemtextNode> {
        for line in raw.lines() {
            match self.state {
                State::Normal => self.parse_normal(line),
                State::Preformatted(_) => self.parse_preformatted(line),
                State::Quote(_) => self.parse_quote(line),
            }
        }
        self.lines
    }

    fn parse_normal(&mut self, line: &'a str) {
        match line {
            s if s.starts_with("=>") => {},
            s if s.starts_with('#') => {},
            s if s.starts_with('*') => {},
            s if s.starts_with('>') => {},
            s if s.starts_with("```") => {},
            s if s.starts_with('<') => {},
            s if s.starts_with(':') => {},
            s if s.starts_with('@') => {},
            s => {},
        }
    }

    fn parse_preformatted(&mut self, line: &'a str) {
    }

    fn parse_quote(&mut self, line: &'a str) {
    }
}

